@@ -11,17 +11,24 @@
 #include "ixion/env.hpp"
 
 #include <cstdlib>
+#include <cstdint>
 
 namespace ixion {
 
 /** Column index type. */
-typedef int col_t;
+using col_t = int32_t;
 
 /** Row index type. */
-typedef int row_t;
+using row_t = int32_t;
 
 /** Sheet index type.*/
-typedef int sheet_t;
+using sheet_t = int32_t;
+
+/**
+ * Integer type that is large enough to store either a row or a column
+ * index.
+ */
+using rc_t = row_t;
 
 /**
  * String ID type.
@@ -30,7 +37,7 @@ typedef int sheet_t;
  * get_string() method of ixion::iface::formula_model_access to get the
  * actual string value.
  */
-typedef unsigned long string_id_t;
+using string_id_t = uint64_t;
 
 /**
  * Special sheet ID that represents a global scope, as opposed to a
@@ -104,28 +111,65 @@ enum class formula_name_resolver_t
 
 /**
  * Formula error types.
+ *
+ * @see get_formula_error_name
  */
-enum class formula_error_t
+enum class formula_error_t : uint8_t
 {
-    no_error = 0,
+    no_error                 = 0,
     ref_result_not_available = 1,
-    division_by_zero = 2,
-    invalid_expression = 3,
-    name_not_found = 4,
-    stack_error = 5,
-    general_error = 999,
+    division_by_zero         = 2,
+    invalid_expression       = 3,
+    name_not_found           = 4,
+    no_range_intersection    = 5,
+    invalid_value_type       = 6,
+
+    no_result_error          = 253, // internal only error
+    stack_error              = 254, // internal only error
+    general_error            = 255, // internal only error
 };
 
-struct IXION_DLLPUBLIC sheet_size_t
+/**
+ * This structure stores a 2-dimensional size information.
+ */
+struct IXION_DLLPUBLIC rc_size_t
 {
     row_t row;
     col_t column;
 
-    sheet_size_t();
-    sheet_size_t(const sheet_size_t& other);
-    sheet_size_t(row_t _row, col_t _column);
+    rc_size_t();
+    rc_size_t(const rc_size_t& other);
+    rc_size_t(row_t _row, col_t _column);
+    ~rc_size_t();
+
+    rc_size_t& operator= (const rc_size_t& other);
 };
 
+/**
+ * This strcuture stores information about grouped formula cells.  All
+ * formula cells belonging to the same group should return the same set of
+ * values.
+ */
+struct IXION_DLLPUBLIC formula_group_t
+{
+    rc_size_t size;
+    uintptr_t identity;
+    bool grouped;
+
+    formula_group_t();
+    formula_group_t(const formula_group_t& r);
+    formula_group_t(const rc_size_t& _group_size, uintptr_t _identity, bool _grouped);
+    ~formula_group_t();
+
+    formula_group_t& operator= (const formula_group_t& other);
+};
+
+/**
+ * Get a string representation of a formula error type.
+ *
+ * @param fe enum value representing a formula error type.
+ * @return string representation of the formula error type.
+ */
 IXION_DLLPUBLIC const char* get_formula_error_name(formula_error_t fe);
 
 }
